﻿package com.hydrotik.go {
	import org.goasap.interfaces.IPlayable;
	import org.goasap.managers.LinearGoRepeater;
	import org.goasap.utils.SequenceCA;
	import org.goasap.utils.SequenceStepCA;
	import org.goasap.utils.customadvance.OnPlayableComplete;
	import org.goasap.utils.customadvance.SequenceAdvance;	

	/**
	 * @author Donovan Adams | Hydrotik | http://blog.hydrotik.com
	 */
	public class HydroSequence extends SequenceCA {
		
		public static const VERSION:String = "HydroTween 0.5.0 rev42 / Go 0.5.1";

		public static const INFO:String = VERSION + " (c) Donovan Adams/Moses Gunesch/Tollman Owens, MIT Licensed.";
		
		private var sequence : SequenceCA = new SequenceCA();

		public function HydroSequence(...actions) {
			super(actions);
			addArrayItem(actions);
		}
		
		public function addItem(...actions) : void
		{
            addArrayItem(actions);
		}
		
		public function addArrayItem(actions:Array):void{
			for (var i:int = 0; i<actions.length; i++) {
				var step:SequenceStepCA = new SequenceStepCA();
				var group:Array = (actions[i] is Array ? actions[i] : [ actions[i] ]);
				for each (var action:Object in group) {
					//if (action.hasOwnProperty("target")==false) { continue; } //left this in so you can see the changes. delete at will!
					var targArray:Array = (action.target is Array ? action.target as Array : [action.target]);
					for each (var targ:Object in targArray) {
						var tween: IPlayable = parseAction(targ, action);
						step.addChild(tween);
						if (action.hasOwnProperty("advance")) {
							var advance: SequenceAdvance = (action["advance"] as SequenceAdvance);
							if (advance) {
								if (advance is OnPlayableComplete) {
									if ((advance as OnPlayableComplete).item==null) (advance as OnPlayableComplete).item = tween;
								}
								step.advance = advance;
							}
						}
					}
				}
				sequence.addStep(step);
            }
		}
		
		
		override public function start():Boolean
		{
			return sequence.start();
		}
  		
		private static function parseAction(target:Object, action:Object):IPlayable{
			var tween : *;
			if(action['target'] != null){
			var pTo:Object = {};
			
				tween = new HydroTween(target,null,NaN,NaN,null,null,null,null,null,null,
					(action.hasOwnProperty("repeater") && action.repeater is LinearGoRepeater ? action.repeater : null));
				for (var prop:String in action) {
					if (prop=="advance" || action[prop] is LinearGoRepeater)
						continue;
					if (tween.hasOwnProperty(prop)) // delay, duration, etc.
						tween[prop] = action[prop];
					else if (prop == "scope") // delay, duration, etc.
						tween.setScope(action[prop]);
					else if (prop == "updater") // delay, duration, etc.
						tween.setUpdate(action[prop]);
					else if (prop == "updaterArgs") // delay, duration, etc.
						tween.setUpdateArgs(action[prop]);
					else if (prop == "func") // delay, duration, etc.
						tween.setClosure(action[prop]);
					else if (prop == "args") // delay, duration, etc.
						tween.setClosureArgs(action[prop]);
					else
						pTo[prop] = action[prop];
				}
				tween.setProps(pTo);
			}else{				
				tween = new HydroTween(new Object, {}, 0, (action['delay'] == null) ? 0 : action['delay'] , null, action['func'], action['updater'], action['args'], action['updaterArgs']);
			}
				
			return tween;
		}
		
	}
}
